package demo.d11;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

/**
 * Ohjelmalla piirretään viiva näyttöön
 * @author Vesa Lappalainen
 * @version 1.0, 24.03.2003
 */
public class PiirtoMalliAWT {

  /**
   * Ikkunaluokka johon piirretään
   */  
  public static class PiirtoIkkuna extends Frame {
    private static final long serialVersionUID = 1L;

    /**
     * Alustetaan ikkunan rajat 
     * @param x ylänurkan x
     * @param y ylänurkan y 
     * @param w ikkunan leveys
     * @param h ikkunan kokrkeus
     */ 
    public PiirtoIkkuna(int x, int y, int w, int h) {
      super();  
      setSize(w,h); setLocation(x,y);
      addWindowListener(new SuljeIkkuna());
    }
    
    /**
     * Ikkunan sulkeva luokka
     */
    protected class SuljeIkkuna extends WindowAdapter {
      /**
       * Sulkemismetodi
       * @param event sulkemistapahtuman tiedot
       */  
      @Override
      public void windowClosing(WindowEvent event) {
        System.exit(0); // NOPMD
      }
    }

    /**
     * Metodi jota kutsutaan aina kun ikkuna haluaa
     * piirtää itsensä uudelleen.
     * Piirretään aina sama viiva
     * @param g Grfiikkaolio jolla tiedot piirtopinnasta
     */
    @Override
    public void paint(Graphics g) {
      Graphics2D g2 = (Graphics2D)g;
      g2.translate(200,150);                       // siirretään origo keskelle
      g2.scale(100,-100);                          // skaalataan
      g2.setStroke(new BasicStroke(1/(float)400)); // kynän paksuus

      g2.draw(new Line2D.Double(-0.9,-0.9,0.9,0.9)); // Tähän piirto
    }

  }

  
  /**
   * Luodaan pääohjelmassa piirto-ikkuna ja laitetaan
   * se näkyville.
   * @param args ei käytössä
   */
  public static void main(String[] args)  {
    PiirtoIkkuna ikkuna = new PiirtoIkkuna(50,50,400,300);
    ikkuna.setVisible(true); // ikkuna.show();
  }
}


