package oliot.aika.rajapinta;

/**
 * Sisäinen tilanvalvonta Aikaan.
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);  a1.toString() === "13:10";
 *  a1.aseta(12,15); a1.toString() === "12:15";
 * </pre>
 */
public class Aika implements IAika {

    private int h=0, m=0;

    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h,int m) {
        aseta(h, m);
    }
    
    /**
     * Alustaa ajan muotoon 00:00
     * @example
     * <pre name="test">
     * new Aika().toString() === "00:00";
     * </pre>
     */
    public Aika() {
        h = 0; m = 0;
      }

    /**
     * Asettaa uuden ajan ja pitää huolen että aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimäärä
     * @param im asetettava minuuttimäärä
     * @return montako vuorokautta jäi yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    @Override
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit väille -59 - 59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit välille 0-23
        
        if (tm<0) { tm += 60; th--; }  //negatiiviset arvot
        if (th<0) { th += 24; vrk--; }
        
        this.h = th; // asetetaan lasketut arvot attribuutteihin
        this.m = tm;
        return vrk; // motako vuorokautta jäi yli
    }

    
    /**
     * Lisää aikaan valitun minuuttimäärän
     * @param lisaMin lisättävä minuuttimäärä
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    @Override
    public void lisaa(int lisaMin)  {
        aseta(getH(), getM() + lisaMin);
    }
    
    
    /**
     * @return ajan minuutit
     */
    @Override
    public int getM() {
        return this.m;
    }


    
    /**
     * @return ajan tunnit
    @Override
    public int getH() {
        return h;
    }
     */
    
    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d",h, m);
    }

    
    /**
     * @example
     * <pre name="test">
     * Aika aika = new Aika(13,37);
     * Aika aika2 = (Aika)aika.clone();
     * aika2.toString() === "13:37";
     * </pre>
     */
    @Override
    public Aika clone() {
        Aika a = new Aika(h, m);
        return a;
    }    
    
    
    /**
     * Tutkitaan aika-luokkaa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Aika a0 = new Aika();
        a0.h = 89;
        System.out.println(a0);
        a0.aseta(12,67);
        System.out.println(a0);
        IAika a1 = new Aika(12,89);
        System.out.println(a1);
    }

    @Override
    public int getH() {
        // TODO Auto-generated method stub
        return 0;
    }
}
