package dynaaminen.intti;

/**
 * Esimerkki dynaamisesta taulukosta
 * @author Vesa Lappalainen @version 1.3 25.2.2019
 * @example
 */
public class Taulukko {
    /** Luokka täyden taulukon poikkeusta varten  */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;

        TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    private int lkm;
    private int[] alkiot;
    
    /**
     * ALustetaan taulukko 
     * @param maxKoko alkukoko
     */
    public Taulukko(int maxKoko) {
        lkm = 0;
        alkiot = new int[maxKoko];
    }
    
    
    /**
     * Lisää uusi alkio
     * @param uusi lisättävä alkio
     * @throws TaulukkoTaysiException jos tila loppuu 
     */
    public void lisaa(int uusi) throws TaulukkoTaysiException {
        if ( lkm >= alkiot.length ) throw new TaulukkoTaysiException("Tila loppu!");
        alkiot[lkm] = uusi;
        lkm++;
    }
    
    
    @Override
    public String toString() {
        StringBuilder tulos = new StringBuilder();
        String ero = "";
        for (int i = 0; i < lkm; i++) {
            int alkio = alkiot[i];
            tulos.append(ero);
            tulos.append(alkio);
            ero = ",";
        }
        return tulos.toString();
    }
    
    
    /**
     * @param i  mihin paikkaan
     * @param uusi mikä arvo
     */
    public void set(int i, int uusi) {
        if ( lkm <= i ) throw new IndexOutOfBoundsException("Laiton indeksi set: " + i);
        alkiot[i] = uusi;
    }
    
    
    /**
     * @param i mistä paikasta
     * @return paikassa oeva arvo
     */
    public int get(int i) {
        if ( i < 0 || lkm <= i ) throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];   
    }
    
    
    /**
     * kokeillaan taulukkoluokkaa 
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taulukko luvut = new Taulukko(8);
        System.out.println(luvut.toString());
        try {
            luvut.lisaa(0);
            luvut.lisaa(2);
            System.out.println(luvut.toString());
            luvut.lisaa(99);
            System.out.println(luvut.toString());
            luvut.lisaa(123);
            luvut.set(6,4);
            int a = luvut.get(3);
            System.out.println(a);
            System.out.println(luvut);
        } catch (IndexOutOfBoundsException e) {
            // e.printStackTrace();
            System.err.println("Tapahtui virhe: " + e.getMessage());
        } catch (TaulukkoTaysiException e) {
            // e.printStackTrace();
            System.err.println("Tapahtui virhe: " + e.getMessage());
        }
    }

}
