/**
 * 
 */
package kerho;

import java.util.List;

/**
 * samoja tekstejä kuin CRC-kortissa
 * @author vesal
 * @version 18.2.2020
 *
 */
public class Kerho {
    
    Jasenet jasenet = new Jasenet();
    Harrastukset harrastukset = new Harrastukset();
    
    
    /**
     * Alustetaan kerho tiedot
     */
    public Kerho() {
        // ei tartte tehä mittään
    }

    
    /**
     * Liästään uusi jäen kerhoon
     * @param jasen lisättävä
     * @throws SailoException jos ei mahu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Kerho kerho = new Kerho();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * aku1.rekisteroi(); aku2.rekisteroi();
     * kerho.getJasenia() === 0;
     * kerho.lisaa(aku1); kerho.getJasenia() === 1;
     * kerho.lisaa(aku2); kerho.getJasenia() === 2;
     * kerho.lisaa(aku1); kerho.getJasenia() === 3;
     * kerho.annaJasen(0) === aku1;
     * kerho.annaJasen(1) === aku2;
     * kerho.annaJasen(2) === aku1;
     * kerho.annaJasen(3) === aku1; #THROWS IndexOutOfBoundsException 
     * kerho.lisaa(aku1); kerho.getJasenia() === 4;
     * kerho.lisaa(aku1); kerho.getJasenia() === 5;
     * kerho.lisaa(aku1);            #THROWS SailoException
     * </pre>
     */
    public void lisaa(Jasen jasen) throws SailoException {
        jasenet.lisaa(jasen);
    }
    
    
    /**
     * 
     * @param har liästtäv harrastus
     * @throws SailoException jos ei onnaa
     */
    public void lisaa(Harrastus har) throws SailoException {
        harrastukset.lisaa(har);
    }
    
    
    /**
     * 
     * @param jasen jäsen jonka harrastukset halutaan
     * @return lista harrastuksista
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }
    
    
    /**
     * Palautaa kerhon jäsenmäärän
     * @return jäsenmäärä
     */
    public int getJasenia() {
        return jasenet.getLkm();
    }
    
    
    /**
     * Palauttaa i:n jäsenen
     * @param i monesko jäsen palautetaan
     * @return viite i:teen jäseneen
     * @throws IndexOutOfBoundsException jos i väärin
     */
    public Jasen annaJasen(int i) throws IndexOutOfBoundsException {
        return jasenet.anna(i);
    }

    
    
    /**
     * Testiohjelma kerhosta
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Kerho kerho = new Kerho();

        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen aku1 = new Jasen(), aku2 = new Jasen();
            aku1.rekisteroi();
            aku1.vastaaAkuAnkka();
            aku2.rekisteroi();
            aku2.vastaaAkuAnkka();

            kerho.lisaa(aku1);
            kerho.lisaa(aku2);
            int id1 = aku1.getTunnusNro();
            int id2 = aku2.getTunnusNro();
            Harrastus pitsi11 = new Harrastus(id1); pitsi11.vastaaPitsinNyplays(id1); kerho.lisaa(pitsi11);
            Harrastus pitsi12 = new Harrastus(id1); pitsi12.vastaaPitsinNyplays(id1); kerho.lisaa(pitsi12);
            Harrastus pitsi21 = new Harrastus(id2); pitsi21.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi21);
            Harrastus pitsi22 = new Harrastus(id2); pitsi22.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi22);
            Harrastus pitsi23 = new Harrastus(id2); pitsi23.vastaaPitsinNyplays(id2); kerho.lisaa(pitsi23);

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < kerho.getJasenia(); i++) {
                Jasen jasen = kerho.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                List<Harrastus> loytyneet = kerho.annaHarrastukset(jasen);
                for (Harrastus harrastus : loytyneet)
                    harrastus.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
    }
}
