package fxKerho;

import java.io.*;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.text.Font;
import kerho.Harrastus;
import kerho.Jasen;
import kerho.Kerho;
import kerho.SailoException;
import fi.jyu.mit.fxgui.ListChooser;

/**
 * Luokka kerhon käyttöliittymän tapahtumien hoitamiseksi.
 * 
 * @author vesal
 * @version 31.12.2015
 * @version 5.1.2015
 */
public class KerhoGUIController implements Initializable {

    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;
    @FXML private Label labelVirhe;
    @FXML ListChooser<Jasen> chooserJasenet;
    @FXML private ScrollPane panelJasen;
    
    private String kerhonnimi = "kelmit";
    
    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();      
    }

    
    @FXML private void handleHakuehto() {
        String hakukentta = cbKentat.getSelectedText();
        String ehto = hakuehto.getText(); 
        if ( ehto.isEmpty() )
            naytaVirhe(null);
        else
            naytaVirhe("Ei osata vielä hakea " + hakukentta + ": " + ehto);
    }
    
    
    @FXML private void handleTallenna() {
        tallenna();
    }
    
    
    @FXML private void handleAvaa() {
        avaa();
    }
    
    
    @FXML private void handleTulosta() {
        TulostusController.tulosta(null);
    } 

    
    @FXML private void handleLopeta() {
        tallenna();
        Platform.exit();
    } 

    
    @FXML private void handleUusiJasen() {
        // Dialogs.showMessageDialog("Vielä ei osata lisätä jäsentä");
        uusiJasen();
    }
    
    
    @FXML private void handleMuokkaaJasen() {
        ModalController.showModal(KerhoGUIController.class.getResource("JasenDialogView.fxml"), "Jäsen", null, "");
    }
    
    
    @FXML private void handlePoistaJasen() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa jäsentä");
    }
    
     
    @FXML private void handleUusiHarrastus() {
        uusiHarrastus();
    }
    

    @FXML private void handleMuokkaaHarrastus() {
        ModalController.showModal(KerhoGUIController.class.getResource("HarrastusDialogView.fxml"), "Harrastus", null, "");
    }
    

    @FXML private void handlePoistaHarrastus() {
        Dialogs.showMessageDialog("Ei osata vielä poistaa harrastusta");
    }
    

    @FXML private void handleApua() {
        avustus();
    }
    

    @FXML private void handleTietoja() {
        // Dialogs.showMessageDialog("Ei osata vielä tietoja");
        ModalController.showModal(KerhoGUIController.class.getResource("AboutView.fxml"), "Kerho", null, "");
    }
    

//===========================================================================================    
// Tästä eteenpäin ei käyttöliittymään suoraan liittyvää koodia    
    
    private Kerho kerho;
    private TextArea areaJasen = new TextArea();
    
    private void alusta() {
        panelJasen.setContent(areaJasen);
        areaJasen.setFont(new Font("Courier New", 12));
        panelJasen.setFitToHeight(true);
        chooserJasenet.clear();
        chooserJasenet.addSelectionListener(e -> naytaJasen());
    }
    
    
    private void uusiJasen() {
        Jasen uusi = new Jasen();
        uusi.rekisteroi();  
        uusi.vastaaAkuAnkka();  // TODO: korvaa oikeasti dialogilla
        try {
            kerho.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
        hae(uusi.getTunnusNro());
    }
    
    

    /** 
     * Tekee uuden tyhjän harrastuksen editointia varten 
     */ 
    public void uusiHarrastus() { 
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject();
        
        if ( jasenKohdalla == null ) return;  
        Harrastus har = new Harrastus();  
        har.rekisteroi();  
        har.vastaaPitsinNyplays(jasenKohdalla.getTunnusNro());  
        try {
            kerho.lisaa(har);
            hae(jasenKohdalla.getTunnusNro());          
        } catch (SailoException e) {
            // TODO muista näyttää dialogi
        }  
    } 
 
    
    
    private void hae(int jnro) {
        chooserJasenet.clear();
    
        int index = 0;
        for (int i = 0; i < kerho.getJasenia(); i++) {
            Jasen jasen = kerho.annaJasen(i);
            if (jasen.getTunnusNro() == jnro) index = i;
            chooserJasenet.add(jasen.getNimi(), jasen);
        }
        chooserJasenet.setSelectedIndex(index); // tästä tulee muutosviesti joka näyttää jäsenen
    }

    
    private void naytaJasen() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject();
    
        if (jasenKohdalla == null) return;
    
        areaJasen.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaJasen)) {
            // jasenKohdalla.tulosta(os);
            tulosta(os,jasenKohdalla);  
        }    
    }

    
    private void tulosta(PrintStream os, Jasen jasen) {
        os.println("----------------------------------------------");
        jasen.tulosta(os);
        os.println("----------------------------------------------");
        List<Harrastus> loytyneet = kerho.annaHarrastukset(jasen);
        for (Harrastus harrastus : loytyneet)
            harrastus.tulosta(os);
    }
    
    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    private void setTitle(String title) {
        ModalController.getStage(hakuehto).setTitle(title);
    }
    
    
    /**
     * Alustaa kerhon lukemalla sen valitun nimisestä tiedostosta
     * @param nimi tiedosto josta kerhon tiedot luetaan
     */
    protected void lueTiedosto(String nimi) {
        kerhonnimi = nimi;
        setTitle("Kerho - " + kerhonnimi);
        //String virhe = "Ei osata lukea vielä";  // TODO: tähän oikea tiedoston lukeminen
        // if (virhe != null) 
        //    Dialogs.showMessageDialog(virhe);
        try {
            kerho.lueTiedosto();
            hae(0);
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }

    
    /**
     * Kysytään tiedoston nimi ja luetaan se
     * @return true jos onnistui, false jos ei
     */
    public boolean avaa() {
        String uusinimi = KerhonNimiController.kysyNimi(null, kerhonnimi);
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }

    
    /**
     * Tietojen tallennus
     */
    private void tallenna() {
        // Dialogs.showMessageDialog("Tallennetetaan! Mutta ei toimi vielä");
        try {
            kerho.tallenna();
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Tallennuksessa tuli virhe: " + e.getMessage());
        }
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }
    

    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    private void avustus() {
        //
    }


    /**
     * Asetetetaan kontrollerin kerhoviite
     * @param kerho mihin viitataan
     */
    public void setKerho(Kerho kerho) {
        this.kerho = kerho;
    }

}
