package kerho;

import java.io.*;

import fi.jyu.mit.ohj2.Mjonot;
import static kanta.HetuTarkistus.rand;

/**
 * Harrastus joka osaa mm. itse huolehtia tunnus_nro:staan.
 *
 * @author Vesa Lappalainen
 * @version 1.0, 22.02.2003
 */
public class Harrastus {
    private int tunnusNro;
    private int jasenNro;
    private String ala;
    private int aloitusvuosi;
    private int tuntiaViikossa;

    private static int seuraavaNro = 1;


    /**
     * Alustetaan harrastus.  Toistaiseksi ei tarvitse tehdä mitään
     */
    public Harrastus() {
        // Vielä ei tarvita mitään
    }


    /**
     * Alustetaan tietyn jäsenen harrastus.  
     * @param jasenNro jäsenen viitenumero 
     */
    public Harrastus(int jasenNro) {
        this.jasenNro = jasenNro;
    }


    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot Harrastukselle.
     * Aloitusvuosi arvotaan, jotta kahdella harrastuksella ei olisi
     * samoja tietoja.
     * @param nro viite henkilöön, jonka harrastuksesta on kyse
     */
    public void vastaaPitsinNyplays(int nro) {
        jasenNro = nro;
        ala = "Pitsin nypläys";
        aloitusvuosi = rand(1900, 2000);
        tuntiaViikossa = rand(0, 60);
    }


    /**
     * Tulostetaan harrastuksen tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(ala + " " + aloitusvuosi + " " + tuntiaViikossa);
    }


    /**
     * Tulostetaan henkilön tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }


    /**
     * Antaa harrastukselle seuraavan rekisterinumeron.
     * @return harrastuksen uusi tunnus_nro
     * @example
     * <pre name="test">
     *   Harrastus pitsi1 = new Harrastus();
     *   pitsi1.getTunnusNro() === 0;
     *   pitsi1.rekisteroi();
     *   Harrastus pitsi2 = new Harrastus();
     *   pitsi2.rekisteroi();
     *   int n1 = pitsi1.getTunnusNro();
     *   int n2 = pitsi2.getTunnusNro();
     *   n1 === n2-1;
     * </pre>
     */
    public int rekisteroi() {
        tunnusNro = seuraavaNro;
        seuraavaNro++;
        return tunnusNro;
    }


    /**
     * Palautetaan harrastuksen oma id
     * @return harrastuksen id
     */
    public int getTunnusNro() {
        return tunnusNro;
    }


    /**
     * Palautetaan mille jäsenelle harrastus kuuluu
     * @return jäsenen id
     */
    public int getJasenNro() {
        return jasenNro;
    }


    /**
     * Asettaa tunnusnumeron ja samalla varmistaa että
     * seuraava numero on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava tunnusnumero
     */
    private void setTunnusNro(int nr) {
        tunnusNro = nr;
        if ( tunnusNro >= seuraavaNro ) seuraavaNro = tunnusNro + 1;
    }


    /**
     * Palauttaa harrastuksen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return harrastus tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Harrastus harrastus = new Harrastus();
     *   harrastus.parse("   2   |  10  |   Kalastus  | 1949 | 22 t ");
     *   harrastus.toString()    === "2|10|Kalastus|1949|22";
     * </pre>
     */
    @Override
    public String toString() {
        return "" + getTunnusNro() + "|" + jasenNro + "|" + ala + "|" + aloitusvuosi + "|" + tuntiaViikossa;
    }


    /**
     * Selvitää harrastuksen tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusnro.
     * @param rivi josta harrastuksen tiedot otetaan
     * @example
     * <pre name="test">
     *   Harrastus harrastus = new Harrastus();
     *   harrastus.parse("   2   |  10  |   Kalastus  | 1949 | 22 t ");
     *   harrastus.getJasenNro() === 10;
     *   harrastus.toString()    === "2|10|Kalastus|1949|22";
     *   
     *   harrastus.rekisteroi();
     *   int n = harrastus.getTunnusNro();
     *   harrastus.parse(""+(n+20));
     *   harrastus.rekisteroi();
     *   harrastus.getTunnusNro() === n+20+1;
     *   harrastus.toString()     === "" + (n+20+1) + "|10|Kalastus|1949|22";
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setTunnusNro(Mjonot.erota(sb, '|', getTunnusNro()));
        jasenNro = Mjonot.erota(sb, '|', jasenNro);
        ala = Mjonot.erota(sb, '|', ala);
        aloitusvuosi = Mjonot.erota(sb, '|', aloitusvuosi);
        tuntiaViikossa = Mjonot.erota(sb, '|', tuntiaViikossa);
    }


    @Override
    public boolean equals(Object obj) {
        if ( obj == null ) return false;
        return this.toString().equals(obj.toString());
    }
    

    @Override
    public int hashCode() {
        return tunnusNro;
    }


    /**
    * Testiohjelma Harrastukselle.
    * @param args ei käytössä
    */
    public static void main(String[] args) {
        Harrastus har = new Harrastus();
        har.vastaaPitsinNyplays(2);
        har.tulosta(System.out);
    }

}