/**
 * 
 */
package kerho;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Hoitelee sen että on monta jäsentä...
 * @author vesal
 * @version 17.2.2020
 *
 */
public class Jasenet {
    
    private static final int MAX_JASENIA   = 5;
    private int              lkm           = 0;
    // private String           tiedostonNimi = "";
    private Jasen[]          alkiot;


    /**
     * Oletusmuodostaja
     */
    public Jasenet() {
        // Attribuuttien oma alustus riittää
        alkiot = new Jasen[MAX_JASENIA]; 
    }


    /**
     * Lisää uuden jäsenen tietorakenteeseen.  Ottaa jäsenen omistukseensa.
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi
     * @throws SailoException jos tietorakenne on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Jasenet jasenet = new Jasenet();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * jasenet.getLkm() === 0;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 1;
     * jasenet.lisaa(aku2); jasenet.getLkm() === 2;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 3;
     * jasenet.anna(0) === aku1;
     * jasenet.anna(1) === aku2;
     * jasenet.anna(2) === aku1;
     * jasenet.anna(1) == aku1 === false;
     * jasenet.anna(1) == aku2 === true;
     * jasenet.anna(3) === aku1; #THROWS IndexOutOfBoundsException 
     * jasenet.lisaa(aku1); jasenet.getLkm() === 4;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 5;
     * jasenet.lisaa(aku1);  #THROWS SailoException
     * </pre>
     */
    public void lisaa(Jasen jasen) throws SailoException   {
        if (lkm >= alkiot.length) throw new SailoException("Liikaa alkioita");
        // TODO: laita tila kasvamaan
        alkiot[lkm] = jasen;
        lkm++;
    }

    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     */
    public Jasen anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }

    
    /**
     * Palauttaa kerhon jäsenten lukumäärän
     * @return jäsenten lukumäärä
     */
    public int getLkm() {
        return lkm;
    }


    /**
     * Tallennetaan jäsentiedosto
     * @throws SailoException jos joku menee pieleen
     */
    public void tallenna() throws SailoException {
        // TODO Auto-generated method stub
        File ftied = new File("kelmit/nimet.dat");
        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath()) ) ) {
            fo.println("Kelmien kerho ry");
            fo.println(alkiot.length);
            for (int i = 0; i < getLkm(); i++) {
                Jasen jasen = anna(i);
                fo.println(jasen.toString()); 
            }
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }

    }
    
 
    /**
     * Luetaan jäsentiedosto
     * @throws SailoException  jos vikaa
     */
    public void lueTiedostosta() throws SailoException {
        File ftied = new File("kelmit/nimet.dat");
        try ( BufferedReader fi = new BufferedReader(new FileReader(ftied.getCanonicalPath())) ) {
            String kokoNimi = fi.readLine(); 
            if ( kokoNimi == null ) throw new SailoException("Kerhon nimi puuttuu");
            String rivi = fi.readLine();
            if ( rivi == null ) throw new SailoException("Maksimikoko puuttuu");
            while ( true ) {
                rivi = fi.readLine();
                if ( rivi == null ) break;
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Jasen jasen = new Jasen();
                jasen.parse(rivi); 
                lisaa(jasen);
            }
        } catch (FileNotFoundException e) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch (IOException e) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        }
        
    }
    

    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Jasenet jasenet = new Jasenet();
        Jasen aku = new Jasen();
        Jasen aku2 = new Jasen();
        aku.rekisteroi();
        aku.vastaaAkuAnkka();
        aku2.rekisteroi();
        aku2.vastaaAkuAnkka();

        try {
            jasenet.lisaa(aku);
            jasenet.lisaa(aku2);
            jasenet.lisaa(aku);
            jasenet.lisaa(aku);
            jasenet.lisaa(aku);
            jasenet.lisaa(aku);
        } catch (SailoException ex) {
            System.err.println(ex.getMessage());
            System.err.flush();
        }
            
            System.out.println("============= Jäsenet testi =================");
            
            for (int i = 0; i < jasenet.getLkm(); i++) {
                Jasen jasen = jasenet.anna(i);
                System.out.println("Jäsen nro: " + i);
                jasen.tulosta(System.out);
            }

        
    }





}
